#include <time.h>
#include <sstream>
#include <deque>
#include "SolverHad.h"

using namespace std;

SolverHad::SolverHad(): tilePosition(0), nextTilePosition(0), goalPosition(0), zeroPosition(0)
{ 
}

SolverHad::~SolverHad() 
{ }



void SolverHad::SetPuzzle(const Puzzle &_puzzle)
{
	puzzle = _puzzle;
	unsigned n = (unsigned)puzzle.GetN();
	
	// + jeste inicializovat masku:
	for(unsigned i=0;i<PUZZLE_MAX_N;++i)
	{
		for(unsigned j=0;j<PUZZLE_MAX_N;++j)
		{
			if(j>=n || i>=n)
				mask[i][j] = mask[j][i] = -1;			// ze se tam nesmi
			else
				mask[i][j] = mask[j][i] = -2;			// volno
		}
	}
}



void SolverHad::SetMaskValue(int position, int value)
{
	if(puzzle.GetN()>0)
	{
		int X, Y;
		puzzle.PositionToCoord(position,X,Y);
		mask[X][Y] = value;
	}
}


void SolverHad::SetMaskValue(int x, int y, int value)
{
	if(puzzle.GetN()>0)
	{
		mask[x][y] = value;
	}
}

int SolverHad::GetMaskValue(int position) const
{
	if(puzzle.GetN()>0)
	{
		int X, Y;
		puzzle.PositionToCoord(position,X,Y);
		return mask[X][Y];
	}
	else
		return 0;

}

string SolverHad::Solve()
{
	// (1) pokud jde vubec resit
	if(!CanSolvePuzzle())
	{
		return "Can't solve this puzzle...";
	}

	cout << "\n";
	// (2) vynulovat pripadne reseni a najit pozici nuly
	solution.clear();
	puzzle.CountZeroPosition();


	// (3) hlavni smycka - rozdel a panuj
	unsigned int n = puzzle.GetN();
	// time_t start,startMini,end;
	// time(&start);
	clock_t start, startMini, end;
	start = clock();

	while(n>3)
	{
		/*
		std::stringstream debugFile;
		debugFile << "C:\\Puzzles\\!debug" << n << ".sbp";
		puzzle.SaveToSbp(debugFile.str());
		*/
		
		SolveFirstRow1();					// prvni polovina 1. radku
		if(!isTileOk(n-1) || !isTileOk(n))
			SolveFirstRow2();				// druha polovina 1. radku	


		SolveFirstCol1();					// prvni polovina 1. sloupce
		if(!isTileOk(2*n+1) || !isTileOk(n+1))
			SolveFirstCol2();					// druha polovina 1. sloupce
		

		if(TrimBoard())
		{
			cout << --n << " ... ";
		}
		else
		{
			cout << "Error! Can't trim board\n ";
			puzzle.Print();
			return "Error! Puzzle hasn't been solved...";
		}
	}
	// (4) pokud uz je to male --> resit "brutalni silou"
	if(n==3)
	{
		startMini = clock();
		string miniSolution = miniPuzzleSearch(puzzle.tiles);
		solution.append(miniSolution);
	}

	end = clock(); 

	double dif1 =  (double)((end - start) * 10 / CLOCKS_PER_SEC) / 10;
	double dif2 =  (double)((end - startMini) * 10 / CLOCKS_PER_SEC) / 10;

	printf("\nSolved in %.1f seconds ( %.1f of it solving 3x3 puzzle)",dif1,dif2);

	// (5) (pripadne overit a) vratit reseni
	return solution;
}
void SolverHad::SolveFirstRow1()
{
	int n = puzzle.GetN();
	int goalCol, goalRow = 0;
	int tileRow, tileCol = 0;
	int nextTileRow = 0;
	int nextTileCol = 0;
	int len = 0;
	string moves = "";
	currentTile = 1;
	bool snakeSolved = false;

	tilePosition = nextTilePosition = goalPosition = zeroPosition = 0;
	
	// prvnich n-2 kamenu
	while(currentTile<n-1)
	{
		#ifdef _DEBUG    
				// puzzle.SaveToSbp("C:\\Puzzles\\!debug.sbp");
		#endif
		zeroPosition = puzzle.GetZeroPosition();
		goalPosition = currentTile-1;
		goalCol = goalPosition;
		tilePosition = puzzle.GetPosition(currentTile);
		puzzle.PositionToCoord(tilePosition,tileRow,tileCol);

		// pokud je n liche -> tak se prvni kamen resi vzdy sam
		if((currentTile==1) && (n%2==1))
		{
			if(isTileOk(currentTile))
			{
				// neni treba nic delat
			}
			else
			{
				SolveTileRow(tileRow,tileCol,0,currentTile-1);
			}
			SetMaskValue(currentTile-1,-1);
			++currentTile;
			continue;
		}
		
		if(isTileOk(currentTile) && isTileOk(currentTile+1))			// vlastne by nemelo nikdy nastat, leda na zacatku
		{
			currentTile +=2;
			continue;
		}

		
		nextTilePosition = puzzle.GetPosition(currentTile+1);
		puzzle.PositionToCoord(nextTilePosition,nextTileRow,nextTileCol);


		// rozhodnout, jaka varianta je nejlepsi
		int cena1 = CountSteps1(zeroPosition,goalPosition,tilePosition,'R',true,false) +
					CountSteps1(goalPosition+1,goalPosition+1,nextTilePosition,'R',true,false);
		int odhad1 = CountSteps2(zeroPosition,goalPosition+1,tilePosition,nextTilePosition,true);
		int odhad2 = CountSteps2(zeroPosition,goalPosition+1,nextTilePosition,tilePosition,false);
		
		#ifdef _DEBUG    
			cout << cena1 << " o1=" <<odhad1  << " o2=" << odhad2 << "\n";
		#endif

		bool pojednom = true;

		if((odhad1<(int)cena1*HAD_KOEF) && (odhad1<=odhad2+2))
		{
			pojednom = false;
			// zkusit udelat jako hada
			StartLog();
			snakeSolved = SolveSnake(tileRow,tileCol,nextTileRow,nextTileCol);
			len = GetLogLengthOptimized();
			StopLog();

			// pokud to prece jen bylo moc dlouhe:
			if(len>cena1-IMPR_KOEF || !snakeSolved )
			{
				pojednom = true;
				  #ifdef _DEBUG    
					// DEBUG kontroly
						cout << "\nReseni pres hada moc dlouhe (nebo nenalezeno): cena=" << cena1 << " vs. snake="
									<< len << "\n";
				  #endif
				RestoreSavedPoint();
				zeroPosition = puzzle.GetZeroPosition();
			}
		}	
		else if((odhad2<(int)cena1*HAD_KOEF) && (odhad2<odhad1-2))
		{
			pojednom = false;
			// zkusit udelat jako hada
			StartLog();
			snakeSolved = SolveSnake(nextTileRow,nextTileCol,tileRow,tileCol);
			len = GetLogLengthOptimized();
			StopLog();

			// pokud to prece jen bylo moc dlouhe:
			if(len>cena1-IMPR_KOEF || !snakeSolved )
			{
				pojednom = true;
				  #ifdef _DEBUG    
					// DEBUG kontroly
						cout << "\nReseni pres hada moc dlouhe (nebo nenalezeno): cena=" << cena1 << " vs. snake="
									<< len << "\n";
				  #endif
				RestoreSavedPoint();
				zeroPosition = puzzle.GetZeroPosition();
			}
		}
		if( pojednom )
		{
			// 1. kamen
			SolveTileRow(tileRow,tileCol,0,currentTile-1);
			SetMaskValue(currentTile-1,-1);
			++currentTile;
			++goalPosition;

			// a hned i 2. kamen
			zeroPosition = puzzle.GetZeroPosition();
			tilePosition = puzzle.GetPosition(currentTile);
			puzzle.PositionToCoord(tilePosition,tileRow,tileCol);
			SolveTileRow(tileRow,tileCol,0,currentTile-1);
			SetMaskValue(currentTile-1,-1);
			++currentTile;
			continue;
		}
		else
		{
			// vyresily se 2 kameny najednou
			currentTile += 2;
		}
		
		
		
	}
	return;
}

void SolverHad::SolveFirstRow2()
{
	// vyresi koncovku 1. rady:
	// predpoklad: kamen n-1 muze byt kdekoliv:

	int n = puzzle.GetN();
	int tileRow, tileCol, zeroRow, zeroCol = 0;
	int goalRow = 0;		
	int goalCol = n-1;		
	currentTile = n-1;
	string moves = "";
	
	zeroPosition = puzzle.GetZeroPosition();
	puzzle.PositionToCoord(zeroPosition,zeroRow,zeroCol);
	
	int tilePosition = puzzle.GetPosition(currentTile);
	puzzle.PositionToCoord(tilePosition,tileRow,tileCol);

	
	// (1) kamen n-1 dostat na posledni misto v 1. radce, pokud tam uz neni:
	if(puzzle[n-1]!=n-1)
	{
		if(puzzle[n-2]==n-1)
		{
			// posunout ho o 1 vys
			MoveZeroToCoord(1,n-1);
			ApplyMoves(MACRO_ROTATEC_DR);
		}
		else if(puzzle[2*n-2]==n-1)	// je ve 2. radku v predposlednim sloupci
		{
			if(zeroPosition==n-2)
			{
				ApplyMoves(MACRO_ROTATEC_UL);
				ApplyMoves("D");			// poodstoupit pro jistotu dolu (jina moznost ostatne ani neni)
			}
			else
			{
				MoveZeroToCoord(2,n-1);
				ApplyMoves("U");		
				ApplyMoves(MACRO_ROTATECC_DR);
			}
		}
		else if(tileCol==n-1)		// je v poslednim sloupci
		{
			MoveZeroUpNextTo(zeroRow,zeroCol,tileRow,tileCol);
			// vytahnout do prvniho radku:
			moves.append("D");
			tileRow--;
			for(;tileRow>goalRow;--tileRow)
			{
				moves.append(MACRO_U_Dl);
			}
			ApplyMoves(moves);
		}
		else					// neni v poslednim sloupci -> std. postup
		{
			SolveTileRow(tileRow,tileCol,1,n-2);
			ApplyMoves(MACRO_ROTATECC_DR);			// aby se dostal doprava nahoru
		}
	}
	// pokud tam uz byl -> tak treba delat nic, pokud neni mezera v 1. radku:
	else if(zeroPosition<n)
		ApplyMoves("D");


	// (2) kamen n dostat pod kamen n-1
	if(puzzle[n-2]==n)
	{
		// v extremnim pripade by so mohlo stat, ze je cely radek hotovy, az na posl. 2 kameny, ktere jsou prohozene
		// v takovem pripade je potreba dostat mezeru na konec 2. radku
		// pokud to neni potreba, pak nasledujici prikaz neudela nic:
		MoveZeroToCoord(1,n-1);
		ApplyMoves(MACRO_ROWEND_SPECIAL2);			// spec. koncovka, kdy je potreba "prehodit" posl. 2 kameny
		return;
	}
	else
	{
		moves.clear();
		currentTile = n;
		tilePosition = puzzle.GetPosition(currentTile);
		puzzle.PositionToCoord(tilePosition,tileRow,tileCol);
		goalRow=1;
		goalCol=n-1;
		
		if(tileCol==n-1)		// je v poslednim sloupci
		{
			puzzle.PositionToCoord(puzzle.GetZeroPosition(),zeroRow,zeroCol);
			MoveZeroUpNextTo(zeroRow,zeroCol,tileRow,tileCol);
			// vytahnout do druheho radku:
			moves.append("D");
			tileRow--;
			for(;tileRow>goalRow;--tileRow)
			{
				moves.append(MACRO_U_Dl);
			}
			moves.append("LU");			// aby se zpod posl. kamene dostala mezera nalevo od nej
			ApplyMoves(moves);
		}
		else if(puzzle[2*n-2]==n)		// je ve 2. radku v predposlednim sloupci
		{
			if(zeroRow==1)
			{
				ApplyMoves("D");			// poodstoupit pro jistotu dolu 
			}
			MoveZeroToCoord(2,n-1);			// dostat se vpravo dolu od kamene 1
			ApplyMoves("UL");				// dostat kamen n do posl. sloupce
		}
		else	// neni v poslednim sloupci -> std. postup
		{
			zeroPosition = puzzle.GetZeroPosition();
			SolveTileRow(tileRow,tileCol,1,n-2);
			ApplyMoves("L");			// aby se dostal do posledniho sloupce
		}
	}


	// (3) uz jen udelat zaverecnou rotaci:
	if(puzzle[2*n-2]==0)
	{
		ApplyMoves(MACRO_ROWEND_STD);
	}
	else
	{
		cout << "\nError in function SolveFirstRow2() (n=" << n << ") : can't do final rotation...";
	}

	return;
}

void SolverHad::SolveFirstCol1()
{
	int n = puzzle.GetN();
	int goalCol, goalRow = 0;
	int tileRow, tileCol = 0;
	int nextTileRow =0;
	int nextTileCol = 0;
	string moves = "";
	currentTile = 1;
	int len = 0;
	bool snakeSolved = false;

	tilePosition = nextTilePosition = goalPosition = zeroPosition = 0;

	
	// prvnich n-3 kamenu
	for(int i=1; i<=n-3; ++i)							// -1 za 1. uz vyplneny radek a -2 za posl 2 dilky
	{
		currentTile = (n*n) - (i*n) + 1;				// pro n=4: 13 ... 9 ... ale 5 uz ne
		goalPosition = currentTile-1;
		zeroPosition = puzzle.GetZeroPosition();
		goalCol = 0;
		goalRow = currentTile/n;
		tilePosition = puzzle.GetPosition(currentTile);
		puzzle.PositionToCoord(tilePosition,tileRow,tileCol);

		// pokud je n sude -> tak prvni pujde vzdy sam:
		if((i==1) && (n%2==0))
		{
			if(isTileOk(currentTile))
			{
				// nic se nedeje
			}
			else
			{
				SolveTileCol(tileRow,tileCol,goalRow,0);
			}
			SetMaskValue(currentTile-1,-1);
			continue;
		}

		if(isTileOk(currentTile) && isTileOk(currentTile-n))
		{
			++i;
			continue;
		}
		

		nextTilePosition = puzzle.GetPosition(currentTile-n);
		puzzle.PositionToCoord(nextTilePosition,nextTileRow,nextTileCol);

		// rozhodnout, jaka varianta je nejlepsi
		int cena1 = CountSteps1(zeroPosition,goalPosition,tilePosition,'U',false,true) +
					CountSteps1(goalPosition+1,goalPosition-n,nextTilePosition,'U',false,true);
		int odhad1 = CountSteps2(zeroPosition,goalPosition+1,tilePosition,nextTilePosition,true);
		int odhad2 = CountSteps2(zeroPosition,goalPosition+1,nextTilePosition,tilePosition,false);
		#ifdef _DEBUG    
			cout << cena1 << " o1=" <<odhad1  << " o2=" << odhad2 << "\n";
		#endif

		bool pojednom = true;

		if((odhad1<(int)cena1*HAD_KOEF) && (odhad1<odhad2))
		{
			pojednom = false;
			// zkusit udelat jako hada
			StartLog();
			snakeSolved = SolveSnake(tileRow,tileCol,nextTileRow,nextTileCol);
			len = GetLogLengthOptimized();
			StopLog();

			// pokud to prece jen bylo moc dlouhe:
			if(len>cena1-IMPR_KOEF || !snakeSolved)
			{
				pojednom = true;
				  #ifdef _DEBUG    
					// DEBUG kontroly
						cout << "\nReseni pres hada moc dlouhe (nebo nenalezeno): cena " << cena1 << " vs. had" 
								<< len << "\n";
				  #endif
				RestoreSavedPoint();
				zeroPosition = puzzle.GetZeroPosition();
			}
		}
		else if((odhad2<(int)cena1*HAD_KOEF) && (odhad2<odhad1-2))
		{
			pojednom = false;
			// zkusit udelat jako hada
			StartLog();
			snakeSolved = SolveSnake(nextTileRow,nextTileCol,tileRow,tileCol);
			len = GetLogLengthOptimized();
			StopLog();

			// pokud to prece jen bylo moc dlouhe:
			if(len>cena1-IMPR_KOEF || !snakeSolved )
			{
				pojednom = true;
				  #ifdef _DEBUG    
					// DEBUG kontroly
						cout << "\nReseni pres hada moc dlouhe (nebo nenalezeno): cena=" << cena1 << " vs. snake="
									<< len << "\n";
				  #endif
				RestoreSavedPoint();
				zeroPosition = puzzle.GetZeroPosition();
			}
		}
		

		if( pojednom )
		{
			// 1. kamen
			SolveTileCol(tileRow,tileCol,goalRow,0);
			SetMaskValue(currentTile-1,-1);
			++i;
			currentTile = (n*n) - (i*n) + 1;

			// a hned i 2. kamen
			zeroPosition = puzzle.GetZeroPosition();
			tilePosition = puzzle.GetPosition(currentTile);
			puzzle.PositionToCoord(tilePosition,tileRow,tileCol);
			SolveTileCol(tileRow,tileCol,goalRow-1,0);
			SetMaskValue(currentTile-1,-1);
			continue;
		}
		else
		{
			++i;		// protoze se vyresily oba dva kameny
		}

		
	}
	return;
}

void SolverHad::SolveFirstCol2()
{
	// vyresi koncovku 1. sloupce:
	// predpoklad: kamen n-1 muze byt kdekoliv:

	int n = puzzle.GetN();
	int tileRow, tileCol, zeroRow, zeroCol = 0;
	int goalRow = 2;		
	int goalCol = 0;		
	currentTile = 2*n+1;
	string moves = "";
	
	zeroPosition = puzzle.GetZeroPosition();
	puzzle.PositionToCoord(zeroPosition,zeroRow,zeroCol);
	
	int tilePosition = puzzle.GetPosition(currentTile);
	puzzle.PositionToCoord(tilePosition,tileRow,tileCol);

	
	// (1) kamen 2*n+1 dostat na 2. nejhornejsi misto v 1. sloupci, pokud tam uz neni:
	if(puzzle[n]!=2*n+1)
	{
		if(puzzle[2*n]==2*n+1)
		{
			// posunout ho o 1 vys
			MoveZeroToCoord(1,1);
			ApplyMoves(MACRO_ROTATEC_UR);
		}
		else if(puzzle[2*n+1]==2*n+1)		// je ve 2. sloupci a 3. radku
		{
			if(zeroPosition==2*n)
			{
				ApplyMoves(MACRO_ROTATEC_DL);
				ApplyMoves("R");			// poodstoupit doprava
			}
			else
			{
				if(zeroCol==1)
					ApplyMoves("R");		// prejit do 3. sloupecku, aby se pohybem vzuru nezrusila pozice kamene 2n
				MoveZeroToCoord(1,2);
				ApplyMoves("L");		
				ApplyMoves(MACRO_ROTATECC_UR);
			}
		}
		else if(tileRow==1)				// uz je v 2. radku
		{
			MoveZeroRightNextTo(zeroRow,zeroCol,tileRow,tileCol);
			// vytahnout do prvniho sloupce:
			ShiftTile(tileCol,0);
		}
		else					// neni v poslednim sloupci -> std. postup
		{
			SolveTileCol(tileRow,tileCol,2,1);
			ApplyMoves(MACRO_ROTATECC_UR);			// aby se dostal na spravnou pozici
		}
	}
	// pokud tam uz byl -> tak netreba delat nic, pokud neni v 1. sloupci mezera
	else if(zeroPosition==2*n)
		ApplyMoves("R");


	// (2) kamen n+1 dostat vpravo vedle kamene 2*n+1
	if(puzzle[2*n]==n+1)
	{
		MoveZeroToCoord(1,1);
		ApplyMoves(MACRO_COLEND_SPECIAL2);			// spec. koncovka, kdy je potreba "prehodit" horni 2 kameny
		return;
	}
	else
	{
		moves.clear();
		currentTile = n+1;
		tilePosition = puzzle.GetPosition(currentTile);
		puzzle.PositionToCoord(tilePosition,tileRow,tileCol);
		goalRow=1;
		goalCol=1;
		
		if(tileRow==1)				// je v druhem radku
		{
			puzzle.PositionToCoord(puzzle.GetZeroPosition(),zeroRow,zeroCol);
			MoveZeroRightNextTo(zeroRow,zeroCol,tileRow,tileCol);
			// vytahnout do druheho sloupce:
			for(;tileCol>goalCol;--tileCol)
			{
				moves.append(MACRO_L_R);
			}
			moves.append("DL");			// aby se mezera dostala pod kamen n+1
			ApplyMoves(moves);
		}
		else					// neni v poslednim sloupci -> std. postup
		{
			zeroPosition = puzzle.GetZeroPosition();
			SolveTileCol(tileRow,tileCol,2,1);
			ApplyMoves("D");			// aby se dostal do posledniho sloupce
		}
	}


	// (3) uz jen udelat zaverecnou rotaci:
	if(puzzle[2*n+1]==0)
	{
		ApplyMoves(MACRO_COLEND_STD);
	}
	else
	{
		cout << "\nError in function SolveFirstCol2() (n=" << n << ") : can't do final rotation...";
	}

	return;
} 

void SolverHad::SolveTileRow(int tileRow,int tileCol, int goalRow, int goalCol)
{

  #ifdef _DEBUG    
	// DEBUG kontroly
	if(this->puzzle[puzzle.GetN()*tileRow+tileCol]!=currentTile)
	{
		cout << "Warning in function SolveTileRow: currentTile(" << currentTile << ") not at specified coords!";
		int tilePos = puzzle.GetPosition(currentTile);
		puzzle.PositionToCoord(tilePos,tileRow,tileCol);
	}
	if(puzzle[zeroPosition]!=0)
	{
		cout << "Warning in function SolveTileRow: zero is not at specified coords!";
		zeroPosition = puzzle.GetZeroPosition();
	}
  #endif

	// if je kamen na sve pozici, tak se jen posune 0 kousek od pozice
	if(tileRow==goalRow && tileCol==goalCol && tileCol<(int)puzzle.GetN())
	{
		MoveZeroToCoord(1,tileCol+1);
		return;
	}

	int zeroRow,zeroCol = 0;
	int n = (int)puzzle.GetN();
	string moves = "";
	bool posunRow = false;
	
	// 1) presun na spravne misto
	puzzle.PositionToCoord(zeroPosition,zeroRow,zeroCol);
	MoveZeroRightNextTo(zeroRow,zeroCol,tileRow,tileCol);

	// 2) dostat do spravneho radku, nebo sloupce
	bool rightSide = tileCol>=goalCol;
	
	// pokud pujdeme zleva, nastavime cilovy radek na 2. odshora, protoze jinak by
	// se mohlo pokazit to, co uz tam je postavene
	// a do 2. radku take v pripade, ze je to predposledni kamen!
	if((!rightSide || currentTile==n-1) && (goalRow==0))
	{
		posunRow = true;
		goalRow++;
	}

	// jit sikmo nahoru, dokud neni ve spravnem sloupci, nebo radku
	while( (tileRow!=goalRow) && (tileCol!=goalCol) )
	{
		// pokud to bylo v prave polovine
		if(rightSide) // jit doleva nahoru
		{
			ApplyMoves(MACRO_UL_R);
			--tileCol;
		}
		else		 // jit doprava nahoru
		{
			ApplyMoves(MACRO_UR_R);
			++tileCol;
		}
		--tileRow;
	}

	// 3) uz jen posun horizontalne / vertikalne
	moves.clear();
	if(tileRow==goalRow)
	{
		if(tileCol>goalCol)				// doleva
		{
			AppendMoves(moves,MACRO_L_R,tileCol-goalCol);		// 
		}
		else							// doprava 
		{
			AppendMoves(moves,MACRO_R_Rd,goalCol-tileCol);
			// ale jestli to bylo doprava, je jeste nutne posunout o 1 nahoru (krome pripadu n-1 a n):
			if(currentTile!=n-1 && currentTile!=n && !rightSide)
				moves.append(MACRO_UP_R);
		}
	}
	else // if(tileCol==goalCol)
	{
		// neni jina moznost, nez ze se musi posunout nahoru
		if(!posunRow)
			AppendMoves(moves,MACRO_UP_R,tileRow-goalRow);
		else
			AppendMoves(moves,MACRO_UP_R,tileRow-goalRow+1);
	}
	ApplyMoves(moves);
	
	return;
}

void SolverHad::SolveTileCol(int tileRow,int tileCol, int goalRow, int goalCol)
{

  #ifdef _DEBUG    
	// DEBUG kontroly
	if(this->puzzle[puzzle.GetN()*tileRow+tileCol]!=currentTile)
	{
		cout << "Warning in function SolveTileCol: currentTile(" << currentTile << ") not at specified coords!";
		int tilePos = puzzle.GetPosition(currentTile);
		puzzle.PositionToCoord(tilePos,tileRow,tileCol);
	}
	if(puzzle[zeroPosition]!=0)
	{
		cout << "Warning in function SolveTileCol: zero is not at specified coords!";
		zeroPosition = puzzle.GetZeroPosition();
	}
  #endif

	// if je kamen na sve pozici, tak se jen posune 0 kousek od pozice
	if(tileRow==goalRow && tileCol==goalCol && tileRow>1)
	{
		MoveZeroToCoord(tileRow-1,1);
		return;
	}

	int zeroRow,zeroCol = 0;
	int n = (int)puzzle.GetN();
	string moves = "";
	bool posunutyCol = false;
	
	// 1) presun na spravne misto
	puzzle.PositionToCoord(zeroPosition,zeroRow,zeroCol);
	MoveZeroUpNextTo(zeroRow,zeroCol,tileRow,tileCol);
		
	// 2) dostat do spravneho radku, nebo sloupce
	bool topSide = tileRow<=goalRow;

		
	// pokud pujdeme zleva, nastavime cilovy radek na 2. odshora, protoze jinak by
	// se mohlo pokazit to, co uz tam je postavene
	if(!topSide && goalCol==0)
	{
		goalCol++;
		posunutyCol = true;
	}

	// jit sikmo doleva, dokud neni ve spravnem sloupci, nebo radku
	while( (tileRow!=goalRow) && (tileCol!=goalCol) )
	{
		// pokud to bylo v horni polovine
		if(topSide) // jit doleva dolu
		{
			ApplyMoves(MACRO_DL_U);
			++tileRow;
		}
		else		 // jit doleva nahoru
		{
			ApplyMoves(MACRO_UL_U);
			--tileRow;
		}
		--tileCol;
	}

	// 3) uz jen posun horizontalne / vertikalne
	moves.clear();
	if(tileCol==goalCol)
	{
		if(tileRow<goalRow)			// dolu
		{
			AppendMoves(moves,MACRO_D_U,goalRow-tileRow);		// 
		}
		else							// nahoru 
		{
			AppendMoves(moves,MACRO_U_U,tileRow-goalRow);
			// ale pokud jsme davali bacha na 1 sloupec, je jeste nutne posunout o 1 doleva: (pze jsme ve 2.sloupci)
			if(posunutyCol)
				moves.append(MACRO_L_U);		
		}
	}
	else // if(tileRow==goalRow)
	{
		if(posunutyCol)
			goalCol--;
		AppendMoves(moves,MACRO_L_U,tileCol-goalCol);					//
	}
	ApplyMoves(moves);
	
	return;
}


bool SolverHad::SolveSnake(int tile1Row, int tile1Col, int tile2Row, int tile2Col)
{
	int n = (int)puzzle.GetN();

  #ifdef _DEBUG    
	// DEBUG kontroly
	if(n<=tile1Row || n<=tile2Row || n<=tile1Col || n<=tile2Col)
	{
		cout << "\n\nWarning in function SolveSnake: coords out of range!";
		return false;
	}
	if(puzzle[zeroPosition]!=0)
	{
		cout << "\n\nWarning in function SolveTileCol: zero is not at specified coords!";
		zeroPosition = puzzle.GetZeroPosition();
	}
  #endif


	// (0) zjistit cisla kamenu
	unsigned int tile1Position = tile1Row*puzzle.GetN() + tile1Col;
	unsigned int tile2Position = tile2Row*puzzle.GetN() + tile2Col;

	int tile1 = puzzle[tile1Position];
	int tile2 = puzzle[tile2Position];

	int zeroRow, zeroCol;
	puzzle.PositionToCoord(zeroPosition,zeroRow,zeroCol);

	bool rightSide = true;


	// (1) nejprve jet k 1. kameni
	//     - mezera na pozici podle toho, jakym smerem je 2. kamen
	if(tile1Col<tile2Col)
	{
		MoveZeroLeftNextTo(zeroRow,zeroCol,tile1Row,tile1Col);
		rightSide = false;
	}
	else
	{
		MoveZeroRightNextTo(zeroRow,zeroCol,tile1Row,tile1Col);
		// pokud pritom doslo k posunu doleva a kamen 1 by se dostal nalevo od kamene 2, je potreba
		//		prehodit mezeru na druhou stranu
		if(tile1Col<tile2Col)
		{
			ApplyMoves("L");
			++tile1Col;
			rightSide = false;
		}
	}
	// mozna nutne znovu prepocitat pozici pro 2. kamen:
	if(puzzle[tile2Position]!=tile2)
	{
		tile2Position = puzzle.GetPosition(tile2);
		puzzle.PositionToCoord(tile2Position,tile2Row,tile2Col);
	}


	// (2) urcit, kam s tim kamenem dojet
	int targetRow = 0;
	int targetCol = 0;

	FindMeetingPoint(tile1Row, tile1Col, tile2Row, tile2Col, targetRow, targetCol);


	// (3) dojet na zadane misto s tim kamenem
	char poziceMezery = rightSide ? 'R' : 'L';
	MoveTileToCoords(tile1Row, tile1Col, targetRow, targetCol, poziceMezery);


	// (4) posunout vedle hlavy hada
	if(!InitSnake(tile1,tile2))
	{
		cout << "\n\nError - cant init snake from tiles " << tile1 << " and " << tile2 << endl;
		return false;
	}

	// (5) zkontrolovat, zda se neporusila pozice minuleho kamene -> to by znamenalo, ze se pohyb pomoci
	//		hada ale stejne zrejme nevyplati
	if(currentTile!=1 && currentTile!=(n*n-n+1))
	{

		int lastTile = currentTile<n ? currentTile-1 : currentTile+n;
		if(!isTileOk(lastTile))
		{
		  #ifdef _DEBUG    
			// cout << "\nDEBUG: Last tile position corrupted. SolveSnake not succesfull.";
		  #endif
			return false;
		}
	}

	if(currentTile>2 && currentTile!=(n*n-n+1) && currentTile!=(n*n-2*n+1))
	{

		int lastTile = currentTile<n ? currentTile-2 : currentTile+2*n;
		if(!isTileOk(lastTile))
		{
		  #ifdef _DEBUG    
			// cout << "\nDEBUG: Last tile position corrupted. SolveSnake not succesfull.";
		  #endif
			return false;
		}
	}


	// (6) naplanovat cestu az "domu"
	if(tile1<n)
		PlanPathRow();
	else
		PlanPathCol();

	
	// (7) jit podel te cesty, pokud to jde
	if(!plan.empty() && !snake.empty())
	{	
		if(plan[0]==snake[0])
			return false;
	}

	return ExecutePath();

}



void SolverHad::FindMeetingPoint(int tile1Row, int tile1Col, int tile2Row, int tile2Col, int& targetRow, int& targetCol)
{
	// Funkce spocita, kam se ma prijit s 1. kamenem sestavovaneho hada,
	// resp. kde jsou cilove souradnice 1. kamene

	targetRow = tile2Row;						// urcite to bude nekde vedle 2. kamene
	targetCol = tile2Col;


	int roz1 = tile1Row - tile1Col;			// urcuje "diagonalu" z leveho horniho do praveho dolniho rohu
	int roz2 = tile2Row - tile2Col;
	int sou1 = tile1Row + tile1Col;			// urcuje "diagonalu" doprava nahoru
	int sou2 = tile2Row + tile2Col;
	
	if( (roz1<roz2) && (sou1<sou2) )			// shora
	{
		targetRow--;
	}
	else if( (sou1>=sou2) && (roz1<=roz2))		// zprava
	{
		targetCol++;
	}
	else if( (sou1>sou2) && (roz1>roz2) )		// zezdola
	{
		targetRow++;
	}
	else // if( (sou1<=sou2) && (roz1>=roz2) )		// zleva
		targetCol--;
}


void SolverHad::PlanPathRow()
{
	// funkce naplanuje cestu hlavy hada az do cilove pozice v 1. rade
	// predpoklad: mezera je vedle hlavy hada

	zeroPosition = puzzle.GetZeroPosition();

	if(snake.empty())
		return;

	int startPosition = snake[0];
	int headPosition = zeroPosition;
	int goalPosition = currentTile;					// POZOR to jo o 1 vlevo, nez je skutecna cilova pozice!
	int n = puzzle.GetN();

	plan.clear();

	int zeroRow = 0;
	int zeroCol = 0;
	int goalRow = 0;
	int goalCol = 0;
	puzzle.PositionToCoord(zeroPosition,zeroRow,zeroCol);
	puzzle.PositionToCoord(goalPosition,goalRow,goalCol);


	// neresitelna pozice:
	if(startPosition==goalPosition-1)
	{
		snake.clear();		// takze nepujde nic delat
		return;
	}


	// UDELAT ZASE NEJAKY FIRST ROW HACK!!!!!
	bool rightSide = zeroCol>=goalCol;
	if(!rightSide)
	{
		// jit do 2. radku, aby se nerozbilo to, co uz je postavene
		goalRow++;
	}

	// ** pokud zbyva jediny tah, neni co resit:
	if(zeroRow==0 && zeroCol==currentTile-1)
	{
		if(snake[0]==goalPosition)
			return;					// tak neni co resit, akorat se pujde 1-krat podel hada
		if(snake[0]==zeroPosition+n)
		{
			if(snake[1]==snake[0]+1)
			{
				// cout << "\n\n||||||||||||||||||||\n\n";
				snake.clear();
				return;		// nevyplati se delat hada!!!
			}
			else
			{
				ApplyMoves("RD");		// had je pod nulou ---> presun do pozice, kde nula je vpravo od hlavy hada
				zeroRow++;
				zeroCol++;
				zeroPosition = puzzle.GetZeroPosition();
			}
		}
	}

	char lastMove = 'X';

	// ***********
	// KROKY SIKMO
	// ***********
	while((zeroRow!=goalRow) && (zeroCol!=goalCol))
	{
		// pojede se sikmo nahoru
		if(zeroCol<goalCol)	// doprava nahoru
		{
			if(lastMove=='X')		// na zacatku ---> urcit, jakym smerem jit
			{
				lastMove = (startPosition==zeroPosition-1) ? 'R' : 'U';
				// pokud byla mezera vpravo od hlavy, tak se zacne vpravo, jinak se jde nahoru
			}

			if(lastMove=='R')
			{
				// tak se pujde vpravo a nahoru, posl. tah bude nahoru
				zeroPosition++;
				plan.push_back(zeroPosition);
				zeroPosition -= n;
				plan.push_back(zeroPosition);

				lastMove = 'U';
			}
			else if(lastMove=='U')
			{
				// tak se pujde nahoru a PAK vpravo, posl. tah bude vpravo
				zeroPosition -= n;
				plan.push_back(zeroPosition);
				zeroPosition++;
				plan.push_back(zeroPosition);

				lastMove = 'R';
			}

			zeroCol++;
			zeroRow--;
		}
		else
		{			  
			// pujde se doleva nahoru
			if(lastMove=='X')		// na zacatku ---> urcit, jakym smerem jit
			{
				lastMove = (startPosition==zeroPosition+1) ? 'L' : 'U';
				// pokud byla mezera vlevo od hlavy, tak se zacne doleva, jinak se jde nahoru
			}
			
			if(lastMove=='L')
			{
				// tak se pujde vlevo a nahoru, posl. tah bude nahoru
				zeroPosition--;
				plan.push_back(zeroPosition);
				zeroPosition -= n;
				plan.push_back(zeroPosition);

				lastMove = 'U';
			}
			else if(lastMove=='U')
			{
				// tak se pujde nahoru a PAK vlevo, posl. tah bude vlevo
				zeroPosition -= n;
				plan.push_back(zeroPosition);
				zeroPosition--;
				plan.push_back(zeroPosition);

				lastMove = 'L';
			}
			
			zeroCol--;
			zeroRow--;
		}
	}

	// ***********
	// KROKY ROVNE
	// ***********
	if(zeroRow==goalRow)
	{
		// tak se pujde po sloupcich:
		while(zeroCol>goalCol)
		{
			--zeroPosition;
			--zeroCol;
			plan.push_back(zeroPosition);
		}
		while(zeroCol<goalCol)
		{
			++zeroPosition;
			++zeroCol;
			plan.push_back(zeroPosition);
		}
	}
	else if(zeroCol==goalCol)
	{
		while(zeroRow>goalRow)		// jina moznost ani neni
		{
			zeroPosition -= n;
			zeroRow--;
			plan.push_back(zeroPosition);
		}
	}

	// ********
	// KONCOVKA
	// ********
	if(!rightSide)
	{
		zeroPosition -= n;
		plan.push_back(zeroPosition);
	}
	// a pridat i posledni bod:
	plan.push_back(--zeroPosition);

	return;
}

void SolverHad::PlanPathCol()
{
	// funkce naplanuje cestu hlavy hada az do cilove pozice v 1. rade
	// predpoklad: mezera je vedle hlavy hada

	zeroPosition = puzzle.GetZeroPosition();

	if(snake.empty())
		return;

	int n = puzzle.GetN();
	int startPosition = snake[0];
	int headPosition = zeroPosition;
	int goalPosition = (currentTile-1)-n;					// o 1 ctverec nahoru, nez je skutecna cilova pozice!

	plan.clear();

	int zeroRow = 0;
	int zeroCol = 0;
	int goalRow = 0;
	int goalCol = 0;
	puzzle.PositionToCoord(zeroPosition,zeroRow,zeroCol);
	puzzle.PositionToCoord(goalPosition,goalRow,goalCol);

	// UDELAT ZASE NEJAKY FIRST COL HACK!!!!!
	bool topSide = zeroRow<=goalRow;
	if(!topSide)
	{
		// jit do 2. sloupce, aby se nerozbilo to, co uz je postavene
		goalCol++;
	}

	// ** pokud zbyva jediny tah, neni co resit:
	if(zeroCol==0 && zeroRow==currentTile/n)
	{
		if(snake[0]==goalPosition)
			return;					// tak neni co resit, akorat se pujde 1-krat podel hada
		if(snake[0]==zeroPosition+1)
		{
			if(snake[1]==snake[0]-n)
			{
				// cout << "\n\n||||||||||||||||||||\n\n";
				snake.clear();
				return;		// nevyplati se delat hada!!!
			}
			else
			{
				ApplyMoves("UR");		// had je vpravo od nuly ---> presun do pozice, kde nula je nad hlavou hada
				zeroRow--;
				zeroCol++;
				zeroPosition = puzzle.GetZeroPosition();
			}
		}
	}

	char lastMove = 'X';

	// ***********
	// KROKY SIKMO
	// ***********
	while((zeroRow!=goalRow) && (zeroCol!=goalCol))
	{
		// pojede se sikmo nahoru
		if(zeroRow>goalRow)	// doleva nahoru
		{
			if(lastMove=='X')		// na zacatku ---> urcit, jakym smerem jit
			{
				lastMove = (startPosition==zeroPosition+1) ? 'L' : 'U';
				// pokud byla mezera vlevo od hlavy, tak se zacne vlevo, jinak se jde nahoru
			}

			if(lastMove=='L')
			{
				// tak se pujde vlevo a nahoru, posl. tah bude nahoru
				zeroPosition--;
				plan.push_back(zeroPosition);
				zeroPosition -= n;
				plan.push_back(zeroPosition);

				lastMove = 'U';
			}
			else if(lastMove=='U')
			{
				// tak se pujde nahoru a PAK vlevo, posl. tah bude vpravo
				zeroPosition -= n;
				plan.push_back(zeroPosition);
				zeroPosition--;
				plan.push_back(zeroPosition);

				lastMove = 'L';
			}

			zeroCol--;
			zeroRow--;
		}
		else
		{			  
			// pujde se doleva dolu
			if(lastMove=='X')		// na zacatku ---> urcit, jakym smerem jit
			{
				lastMove = (startPosition==zeroPosition+1) ? 'L' : 'D';
				// pokud byla mezera vlevo od hlavy, tak se zacne doleva, jinak se jde dolu
			}
			
			if(lastMove=='L')
			{
				// tak se pujde vlevo a dolu, posl. tah bude dolu
				zeroPosition--;
				plan.push_back(zeroPosition);
				zeroPosition += n;
				plan.push_back(zeroPosition);

				lastMove = 'D';
			}
			else if(lastMove=='D')
			{
				// tak se pujde dolu a PAK vlevo, posl. tah bude vlevo
				zeroPosition += n;
				plan.push_back(zeroPosition);
				zeroPosition--;
				plan.push_back(zeroPosition);

				lastMove = 'L';
			}
			
			zeroCol--;
			zeroRow++;
		}
	}

	// ***********
	// KROKY ROVNE
	// ***********
	if(zeroCol==goalCol)
	{
		// tak se pujde po radcich:
		while(zeroRow>goalRow)
		{
			zeroPosition -= n;
			--zeroRow;
			plan.push_back(zeroPosition);
		}
		while(zeroRow<goalRow)
		{
			zeroPosition += n;
			++zeroRow;
			plan.push_back(zeroPosition);
		}
	}
	else if(zeroRow==goalRow)
	{
		while(zeroCol>goalCol)		// jina moznost ani neni
		{
			zeroPosition--;
			zeroCol--;
			plan.push_back(zeroPosition);
		}
	}

	// ********
	// KONCOVKA
	// ********
	if(!topSide)
	{
		zeroPosition--;
		plan.push_back(zeroPosition);
	}
	// a pridat i posledni bod:
	zeroPosition += n;
	plan.push_back(zeroPosition);

	return;
}


bool SolverHad::ExecutePath()
{
	//  -- prvni dva kroku zkusit udelat, jesti to porad nebude vedle hada

	if(snake.empty())
		return false;

	if(plan.size()>2)			// pro kratsi plany nelze provadet
	{
		// druhy krok planu jestli nahodou neni porad vedle stavajici hlavy:
		int n = puzzle.GetN();
		if( (plan[1]==snake[0]-1 || plan[1]==snake[0]-n || plan[1]==snake[0]+1 || plan[1]==snake[0]+n) 
			  &&
			 (plan[1]!=snake[1]) )
		{
			// tak ty dva tahy rovnou udelat a nehybat kvuli tomu celym hadem:
			zeroPosition = puzzle.GetZeroPosition();
			string moves = "";
			moves.append(1,DetermineDirection(zeroPosition,plan[0],n));
			moves.append(1,DetermineDirection(plan[0],plan[1],n));
			ApplyMoves(moves);

			// prvni dva tahy vymazat:
			zeroPosition = puzzle.GetZeroPosition();
			plan.erase(plan.begin());
			plan.erase(plan.begin());

		}
	}

	// 1. krok
	MoveAlongSnake();



	// dalsi kroky:
	int pos = 0;
	string moves = "";
	for(unsigned step=0; step<plan.size();++step)
	{
		pos = plan[step];
		
		// jit na tu pozici:
		moves = FindMovesTo(pos);
		ApplyMoves(moves);

		// a podel hada
		MoveAlongSnake();
		// break;
	}

	return true;
}

bool SolverHad::InitSnake(int tile1, int tile2)
{
	zeroPosition = puzzle.GetZeroPosition();
	
	bool hlava1 = ( (tile1==tile2-1) || (tile1==tile2+puzzle.GetN()) )? true : false;			// jestli hlavu hada tvori 1. kamen, nebo ne
	
	int pos1 = puzzle.GetPosition(tile1);
	int pos2 = puzzle.GetPosition(tile2);

  #ifdef _DEBUG    
	// DEBUG kontroly
	// - ze je mezera bud vlevo nebo vpravo od tile1
	if(zeroPosition!=pos1+1 && zeroPosition!=pos1-1)
	{
		cout << "\n\nError in function InitSnake: unexpected zero position.";
		return false;
	}
  #endif

	snake.clear();

	if(!hlava1)
	{
		// nutno presunout vedle tile2
		const char directionZero = DetermineDirection(zeroPosition,pos1,puzzle.GetN());
		const char direction2 = DetermineDirection(pos1,pos2,puzzle.GetN());
		if(directionZero=='R')
		{
			if(direction2=='U')
				ApplyMoves("U");
			else if(direction2=='D')
				ApplyMoves("D");
			else if(direction2=='R')
			{
				ApplyMoves("RR");
				pos1--, pos2--;
			}
			else
			{
				cout << "\n\nError in function InitSnake: unexpected tile2 position.";
				return false;
			}
		}
		else if(directionZero=='L')
		{
			if(direction2=='U')
				ApplyMoves("U");
			else if(direction2=='D')
				ApplyMoves("D");
			else if(direction2=='L')
			{
				ApplyMoves("LL");
				pos1++, pos2++;
			}
			else
			{
				cout << "\n\nError in function InitSnake: unexpected tile2 position.";
				return false;
			}
		}
		else
		{
			cout << "\n\nError in function InitSnake: unexpected zero position.";
			return false;
		}

		// a ted jen inicializovat hada:
		snake.push_back(pos2);
		snake.push_back(pos1);
	}
	else
	{
		snake.push_back(pos1);
		snake.push_back(pos2);
	}

	// test, jestli by stalo za to zkusit i dalsi kamen:
	/* 
	if(tile1<puzzle.GetN())
	{
		int pos3 = hlava1 ? puzzle.GetPosition(tile1+2) : puzzle.GetPosition(tile1+1);
		int hrow,hcol,trow,tcol;
		puzzle.PositionToCoord(pos1,hrow,hcol);
		puzzle.PositionToCoord(pos3,trow,tcol);

		if((trow<=hrow) && ( ((tcol<=hcol) && (tcol>=tile1)) || ((tcol>=hcol) && (tcol<=tile1))))
			vyplati++;
		else
			nevyplati++;
	}
	*/


	return true;
}


string SolverHad::FindMovesTo(int goalPosition)
{
	int n = puzzle.GetN();
	if(!(n>=0))
		return "";
	
	zeroPosition = puzzle.GetZeroPosition();
	int startPosition = zeroPosition;
	SetMaskValue(zeroPosition,-1);
	// prohledavanim do sirky najit posloupnost tahu
	deque<int> okraj;				// pozice
	vector<int> prozkoumane;		// do kterych se slo - na konci "vycistit"
	deque<int> cesta;
	string moves = "";

	okraj.push_back(zeroPosition);
	prozkoumane.push_back(zeroPosition);
	bool nalezeno = false;
	int next = 0;
	int actual = 0;

	while(!nalezeno)
	{
		if(okraj.empty())
				break;
		
		actual = okraj[0];
		okraj.pop_front();

		// je to konec?
		if(actual==goalPosition)
		{
			nalezeno = true;
		}
		else
		{
			// neni, takze projit vsechny 4 sousedy
			if(actual>=n)
			{
				next = actual-n;
				if(GetMaskValue(next)==-2)					// volno a jeste se tam nebylo
				{
					okraj.push_back(next);
					prozkoumane.push_back(next);
					SetMaskValue(next,actual);
				}
			}
			if(actual%n!=n-1)
			{
				next = actual+1;
				if(GetMaskValue(next)==-2)					// volno a jeste se tam nebylo
				{
					okraj.push_back(next);
					prozkoumane.push_back(next);
					SetMaskValue(next,actual);
				}
			}
			if(actual%n!=0)
			{
				next = actual-1;
				if(GetMaskValue(next)==-2)					// volno a jeste se tam nebylo
				{
					okraj.push_back(next);
					prozkoumane.push_back(next);
					SetMaskValue(next,actual);
				}
			}
			if(actual<n*n-n)
			{
				next = actual+n;
				if(GetMaskValue(next)==-2)					// volno a jeste se tam nebylo
				{
					okraj.push_back(next);
					prozkoumane.push_back(next);
					SetMaskValue(next,actual);
				}
			}
		}
	}

	if(nalezeno)
	{
		// super, nalezla se cesta
		// pujde se pozpatku:
		
		while(GetMaskValue(actual)!=-1)
		{
			cesta.push_front(actual);
			actual = GetMaskValue(actual);
		}

		// ted brat tu cestu postupne:
		actual = startPosition;
		while(!cesta.empty())
		{
			next = cesta[0];
			moves.append(1, DetermineDirection(actual,next,n ));
			actual = next;
			cesta.pop_front();
		}

		// uklidit:
		vector<int>::iterator pr_it = prozkoumane.begin();
		while(pr_it!=prozkoumane.end())
		{
			SetMaskValue(*pr_it,-2);
			++pr_it;
		}

		return moves;


	}
	else
	{
		cout << "Error in FindMovesTo - failed to reach final destination!";
		return "";
	}

}

void SolverHad::MoveZeroToCoord(int row, int col)
{
	// fce presune mezeru na zadane souradnice BEZ OHLEDU na zbytek kemenu na desce
	int n = (int)puzzle.GetN();

  #ifdef _DEBUG    
	// DEBUG kontroly
	if(n<=row || n<=col)
	{
		cout << "Warning in function MoveZeroToCoord: coordinates(" << row << "," << col << ") out of range!";
		return;
	}
	
  #endif

	if(n==0)
		return;

	string moves;
	zeroPosition = puzzle.GetZeroPosition();
	int zeroRow, zeroCol;
	puzzle.PositionToCoord(zeroPosition,zeroRow,zeroCol);

	// jakym smerem vertikalne to bude:
	char moveV = (zeroRow>row) ? 'U' : 'D';
	int kroku = abs(zeroRow-row);
	moves.append(kroku,moveV);

	// jak horizontalne:
	char moveH = (zeroCol>col) ? 'L' : 'R';
	kroku = abs(zeroCol-col);
	moves.append(kroku,moveH);

	ApplyMoves(moves);

	zeroPosition = n*row + col;
	return;
}


void SolverHad::MoveTileToCoords(int tileRow, int tileCol, int targetRow, int targetCol, char poziceMezery)
{
	// funkce, ktere presune kamen tile1 ze souradnic [tile1Row, tile1Col] na [targetRow, targetCol]
	//		bez ohledu na zbytek desky
	//		- pouziva se pro "sestaveni hada"

	// predpoklad(y):	* mezera je ve smeru poziceMezery od kamene!
	//					* poziceMezery je bud 'R' nebo 'L'

	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	// Q: Proc se nepouziva tato funkce ve vsech castech, kde se hybe s kamenem??? (napr. pri reseni 1. rady??)
	// A: Prave pro to, ze uplne ignoruje zbytek herni desky, pri reseni prvni rady je
	//		treba dodelat ruzne "hacky" na osetreni uz sestavene casti (aby se nerozbila)
	//		- pri sestavovani hada se do uz sestavene casti 1.radky (sloupce) nezasahuje, takze neni potreba to
	//		  v teto fci osetrovat
	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	
  #ifdef _DEBUG    
	// DEBUG kontroly
	int zp = puzzle.GetN()*tileRow + tileCol;
	poziceMezery=='R' ? zp++ : zp--;
	if(puzzle[zp]!=0)
	{
		cout << "\n\nError in function MoveTileToCoords: zero is not at expected coordinates!";
		cout << "\n\t tileRow = " << tileRow << "\t tileCol = " << tileCol << "\t poziceMezery = " << poziceMezery;
		puzzle.Print();
		return;
	}		
  #endif

	int n = puzzle.GetN();

	// osetreni, kdyz se jede do prvniho nebo posledniho sloupce:
	if(targetCol==0 && poziceMezery=='L')
	{
		ApplyMoves("R");
		poziceMezery = 'R';
		tileCol--;
	}
	if(targetCol==n-1 && poziceMezery=='R')
	{
		ApplyMoves("L");
		poziceMezery = 'L';
		tileCol++;
	}

	if(poziceMezery!='R' && poziceMezery!='L')
	{
		cout << "\n\nError int function MoveTileToCoords: unexpected value of 'poziceMezery' : " << poziceMezery;
		return;
	}

	zeroPosition = puzzle.GetZeroPosition();

	// nejdriv pohyb sikmo, dokud to jde
	while((tileRow!=targetRow) && (tileCol!=targetCol))
	{
		if(tileRow<targetRow && tileCol<targetCol)
		{
			poziceMezery=='L' ? ApplyMoves(MACRO_DR_L) : ApplyMoves(MACRO_DR_R);
			++tileRow, ++tileCol;
		}
		else if(tileRow<targetRow && tileCol>targetCol)
		{
			poziceMezery=='L' ? ApplyMoves(MACRO_DL_L) : ApplyMoves(MACRO_DL_R);
			++tileRow, --tileCol;
		} 
		else if(tileRow>targetRow && tileCol>targetCol)
		{
			poziceMezery=='L' ? ApplyMoves(MACRO_UL_L) : ApplyMoves(MACRO_UL_R);
			--tileRow, --tileCol;
		}
		else if(tileRow>targetRow && tileCol<targetCol)
		{
			poziceMezery=='L' ? ApplyMoves(MACRO_UR_L) : ApplyMoves(MACRO_UR_R);
			--tileRow, ++tileCol;
		}
	}
	
	// pak pohyb rovne
	string macro = "";
	if(tileRow==targetRow)		// HORIZONTALNE
	{
		if(tileCol<targetCol)	// pujde se doprava
		{
			if(poziceMezery=='L')
			{
				macro = tileRow==n-1 ? MACRO_R_Lu : MACRO_R_Ld;
			}
			else
			{
				macro = tileRow==n-1 ? MACRO_R_Ru : MACRO_R_Rd;
			} // mezera vpravo a jde se doprave
		}
		else // pujde se doleva
		{
			if(poziceMezery=='L')
			{
				macro = tileRow==n-1 ? MACRO_L_Lu : MACRO_L_Ld;
			}
			else
			{
				macro = tileRow==n-1 ? MACRO_L_Ru : MACRO_L_Rd;
			}
		}
		for(int i=0; i<abs(tileCol-targetCol); ++i)
			ApplyMoves(macro);
	}
	else if(tileCol==targetCol)		// HORIZONTALNE
	{
		if(tileRow<targetRow)		// pujde se dolu
		{
			macro = poziceMezery=='L' ? MACRO_D_L : MACRO_D_R;
		}
		else						// pujde se nahoru
		{
			macro = poziceMezery=='L' ? MACRO_U_L : MACRO_U_R;
		}
		for(int i=0; i<abs(tileRow-targetRow); ++i)
			ApplyMoves(macro);
	}
}


void SolverHad::MoveAlongSnake()
{
	// provede se pohyb "podel" hada
	
	// predpoklad: mezera je nekde na pozici vedle hlavy hada
	int startPos = zeroPosition = puzzle.GetZeroPosition();
	string moves = "";

	int actPos = zeroPosition;
	int nextPos = 0;
	// projit celeho hada
	for(unsigned i=0; i<snake.size(); ++i) 
	{
		nextPos = snake[i];
		char move = DetermineDirection(actPos,nextPos,puzzle.GetN());
		if(move=='X')
		{
			cout << "\nError in function MoveAlongSnake - can't determine next step. P1=" << actPos << " P2=" << nextPos;
			return;
		}
		else
		{
			moves.append(1,move);
			snake[i] = actPos;
			
			// jeste si oznacit, kde ten had je
			SetMaskValue(actPos,-1);

			//
			actPos = nextPos;
		}
	}

	ApplyMoves(moves);
	zeroPosition = actPos;
	SetMaskValue(zeroPosition,-2);

}


void SolverHad::MaSouseda(int tileNumber,int tilePosition)
{
	vector<SOUSED> sousedi;

	puzzle.GetNeighbours(tilePosition,sousedi);
}


int SolverHad::CountSteps1(int poziceStart, int poziceCil, int poziceKamen, const char smer,
						   bool prvniRadek, bool prvniSloupec)
{
	if(this->puzzle.tiles.empty() || this->puzzle.GetN()<2)
		return -1;
	if((smer!='U') && (smer!='R') && (smer!='L') && (smer!='D'))
		return -1;

	int kroku = 0;

	int startRow, startCol, cilRow, cilCol, kamenRow, kamenCol;
	int targetRow, targetCol;			// aktualni cil "mezery"
	puzzle.PositionToCoord(poziceStart, startRow, startCol);
	puzzle.PositionToCoord(poziceKamen, kamenRow, kamenCol);
	puzzle.PositionToCoord(poziceCil, cilRow, cilCol);


	// nejdriv dostat mezeru na zadane sousedni policko
	//		-- ale bacha na posun kamene
	FindTargetCoord(startRow, startCol, kamenRow, kamenCol, smer, targetRow, targetCol);

	kroku += abs(startRow-targetRow) + abs(startCol-targetCol);

	if(poziceCil==poziceKamen)
		return kroku;			// zadne dalsi kroky se delat nebudou


	// ted (mozna) par kroku sikmo 
	// !! POZOR - ted uz se pocita od skutecne pozice kamene !!
	int rozdilX = abs(kamenRow-cilRow);
	int rozdilY = abs(kamenCol-cilCol);
	// kroku sikmo bude [mensi_rozdil] a pak rovne [vetsi_rozdil]-[mensi_rozdil]:
	kroku += min(rozdilX,rozdilY)*KROKU_1_SIKMO + abs(rozdilX-rozdilY)*KROKU_1_ROVNE;


	// v nekterych pripadech nelze jit sikmo az do prvniho radku! (resp. sloupce)
	if(    (prvniRadek && ( (kamenRow+kamenCol)<=(cilRow+cilCol) ))  
        || (prvniSloupec && ( (kamenRow-kamenCol)>=(cilRow-cilCol) )) )
	{
		kroku = kroku - KROKU_1_SIKMO + 2*KROKU_1_ROVNE;
	}

	return kroku;
}


int SolverHad::CountSteps2(int poziceStart, int poziceCil, int poziceK1, int poziceK2, bool hlava1)
{
	// funkce spocita pocet kroku (resp. odhad), ktery je nutny na
	// sestaveni hada z kamenu na pozicich poziceK1 a poziceK2 (hlava hada se urcti
	// podle hodnoty parametru hlava1: TRUE=K1 bude hlava) a dopraveni ho na
	// cilovou pozici:
	//		to znamena, ze HLAVA HADA BUDE NA POZICI poziceCil
	//		druhy kamen ale muze byt umisten jakkoliv
	//
	//      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	//		--> proto je treba volat tuto fci a jako parametr poziceCil predat
	//			zamyslenou pozici 2. kamene a k vysledku funkce pripocist jeste
    //			par kroku na posl. krok hada (odhad neni presny)
	//		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	int N = this->puzzle.GetN();

	if(this->puzzle.tiles.empty() || N<2)
		return -1;
	
	int kroku, krokuEtapa0, krokuEtapa1, krokuEtapa2 = 0;					// pocet kroku
	int krokuRovne, krokuSikmo = 0;
	int startRow, startCol, cilRow, cilCol, kamen1Row, kamen1Col, kamen2Row, kamen2Col;
	
	puzzle.PositionToCoord(poziceStart, startRow, startCol);
	puzzle.PositionToCoord(poziceCil, cilRow, cilCol);
	puzzle.PositionToCoord(poziceK1, kamen1Row, kamen1Col);
	puzzle.PositionToCoord(poziceK2, kamen2Row, kamen2Col);


	// (0) nejprve od startu tesne vedle kamene K1
	krokuEtapa0 = abs(startRow-kamen1Row) + abs(startCol-kamen1Col) - 1;	// -1 pze se jde VEDLE, ne na tu pozici


	// (1) ted se dostat vedle kamene K2
	//
	// je treba ale rozlisovat kroky sikmo a kroku stranou!
	
	int rozdilX = abs(kamen1Row-kamen2Row);
	int rozdilY = abs(kamen1Col-kamen2Col);

	if(rozdilX==rozdilY)					
	{
		// pak jsou na stejne "uhlopricce"
		krokuSikmo = rozdilX-1;
		krokuRovne = 1;
	}
	else
	{
		krokuSikmo = min(rozdilX,rozdilY);
		krokuRovne = max(rozdilX,rozdilY) - krokuSikmo - 1;			
	}

	krokuEtapa1 = krokuRovne*KROKU_1_ROVNE + krokuSikmo*KROKU_1_SIKMO;
	// !!! to je HORNI ODHAD POCTU KROKU !!!
	// ve skutecnosti se mozna udela kroku mene, protoze u posledniho
	// "makro-kroku" nebude potreba (ani mozne) udelat vsech 5 pohybu,
	// ale vystaci treba 3


	// (2) ted hada dostat do cile
	//
	// presneji hlavu hada na pozici poziceCil

	// je potreba urcit pozici hlavy:
	if(hlava1)
	{
		int roz1 = kamen1Row - kamen1Col;
		int roz2 = kamen2Row - kamen2Col;
		int sou1 = kamen1Row + kamen1Col;
		int sou2 = kamen2Row + kamen2Col;
		if( (roz1<=roz2) && (sou1<sou2) )			// shora
		{
			kamen2Row--;
		}
		else if( (sou1>=sou2) && (roz1<roz2))		// zprava
		{
			kamen2Col++;
		}
		else if( (sou1>sou2) && (roz1>=roz2) )		// zezdola
		{
			kamen2Row++;
		}
		else // if( (sou1<=sou2) && (roz1>roz2) )		// zleva
			kamen2Col--;
	}

	// spocitat vzdalenost od hlavy k cili
	rozdilX = abs(kamen2Row-cilRow);
	rozdilY = abs(kamen2Col-cilCol);

	krokuSikmo = min(rozdilX,rozdilY);
	krokuRovne = max(rozdilX,rozdilY) - krokuSikmo;			

	// na zacatku jeste potreba udelat az 3 kroky, aby se mezera dostala vedle hlavy hada
	krokuEtapa2 = 3 + krokuRovne*KROKU_2_ROVNE + krokuSikmo*KROKU_2_SIKMO;

	kroku = krokuEtapa0 + krokuEtapa1 + krokuEtapa2;
	return kroku;
}

void SolverHad::FindTargetCoord(int startRow, int startCol, int& cilRow, int& cilCol, const char smer,
								int& targetRow, int& targetCol)
{
	/*
	Funkce pocita, kde skonci "mezera", pokud zacina na souradnicich [startR,startC],
	cilovy kamen ja na [cilR,cilC]
	a skoncit ma vedle nej ve smeru smer
	*/
	
	if((smer!='U') && (smer!='R') && (smer!='L') && (smer!='D'))
		return;

	targetRow = cilRow;
	targetCol = cilCol;
	
	// rozhodnout podle zadaneho smeru
	if(smer=='R')
	{
		if(startCol<cilCol)
		{
			// tak je to oukej, nejdriv se jde do zadaneho radku, a pak doprava, cimz dojde k odsunuti kamene
			// o 1 policko vlevo
			--cilCol;
		}
		else
		{
			++targetCol;
		}
	}
	else if(smer=='L')
	{
		if(startCol>cilCol) 
		{
			++cilCol;
		}
		else
		{
			--targetCol;
		}
	}
	else if(smer=='U')
	{
		if(startRow>cilRow)
		{
			++cilRow;
		}
		else
		{
			--targetRow;
		}
	}
	else if(smer=='D')
	{
		if(startRow<cilRow)
		{
			--cilRow;
		}
		else
		{
			++targetRow;
		}
	}
	return;

}


bool SolverHad::TrimBoard()
{
	if(puzzle.TrimBoard())
	{
		unsigned n = (unsigned)puzzle.GetN();
	
		// + jeste inicializovat masku:
		for(unsigned i=0;i<PUZZLE_MAX_N;++i)
		{
			for(unsigned j=0;j<PUZZLE_MAX_N;++j)
			{
				if(j>=n || i>=n)
					mask[i][j] = mask[j][i] = -1;			// ze se tam nesmi
				else
					mask[i][j] = mask[j][i] = -2;			// volno
			}
		}
		return true;

	}
	else
		return false;

}




void SolverHad::Test()
{
	Solve();
	return;

	int zeroRow, zeroCol;
	puzzle.Print();
	zeroPosition = puzzle.GetZeroPosition();
	puzzle.PositionToCoord(zeroPosition,zeroRow,zeroCol);
	
	int t1 = puzzle.GetPosition(1);
	int t2 = puzzle.GetPosition(2);

	int r1,c1,r2,c2;
	puzzle.PositionToCoord(t1,r1,c1);
	puzzle.PositionToCoord(t2,r2,c2);

	CountSteps2(zeroPosition,1,t1,t2,true);
	CountSteps2(zeroPosition,1,t2,t1,false);
	SolveSnake(r1,c1,r2,c2);
	
	puzzle.Print();

	return;

	zeroPosition = puzzle.GetZeroPosition();
	 t1 = puzzle.GetPosition(1);
	 t2 = puzzle.GetPosition(2);

	snake.clear();
	snake.push_back(t1);
	snake.push_back(t2);

	MoveAlongSnake();

}