#include <time.h>
#include <sstream>
#include <iostream>
#include <fstream>

#include "PuzzleInputFce.h"
#include "Puzzle.h"
#include "miniPuzzle.h"
#include "SolverPRT.h"
#include "SolverHad.h"

using namespace std;

const unsigned int MAX_N = PUZZLE_MAX_N;


void printHelp()
{
	cout << "\n ***(n^2-1) Puzzles***\n\n";
	cout << "List of commands:";
	cout << "\n\thelp\t...\tprint this help";
	cout << "\n\tcls\t...\tclear screen";
	cout << "\n\tload\t...\tload puzzle from file";
	cout << "\n\tsave\t...\tsave puzzle";
	cout << "\n\tnew\t...\tnew puzzle n x n (tiles in their goal positions)";
	cout << "\n\trand\t...\tnew puzzle n x n (random tiles positions)";
	cout << "\n\tprint\t...\tprint current puzzle";
	cout << "\n\trmove\t...\tdo n random moves";
	cout << "\n\tsolve\t...\tsolve current puzzle (Parberry)";
	cout << "\n\tsolve2\t...\tsolve current puzzle ('Snakes')";
	cout << "\n\tapply\t...\tapply solution to current puzzle";
	cout << "\n\tsolution\t...\tprint solution";
	
	cout << "\n\texit\t...\texit the program";
}

void loadPuzzle(Puzzle &puzz)
{
	cout << "\nInsert file name : ";
	string filename;
	cin >> filename;
	
	puzz.LoadFromSbp(filename);
}

void savePuzzle(Puzzle &puzz)
{
	cout << "\nInsert file name : ";
	string filename;
	cin >> filename;
	
	puzz.SaveToSbp(filename);
}

void savePuzzle(Puzzle &puzz, string filename)
{
	puzz.SaveToSbp(filename);
}

void randomizePuzzle(Puzzle& puzz, int n)
{
	puzz.ResetTiles(n);
	puzz.SetN(n);
	int iteraci = 0;

	do
	{
		// vytvori se seznam vsech moznych nepouzitych kamenu:
		vector<int> tiles;
		int tile = 0;
		for(int i=0;i<n*n;++i)
			tiles.push_back(i);

		// pro kazdou pozici nahodne vybrat jeste volny kamen
		vector<int>::iterator tilesIt = tiles.begin();
		for(int position=0;position<n*n;++position)
		{
			tilesIt=tiles.begin();
			// kolik je jeste volnych kamenu
			int pos = tiles.size();
			
			// nahodne nejaky vybrat
			int randomPosition = myRand(pos-1);					// -1, pze se indexuje od 0 (pri dvou prvcich chci 0 nebo 1)

			tilesIt += randomPosition;
			tile = *tilesIt;

			// ten priradit
			puzz.tiles[position] = tile;

			// ...a vymazat ze seznamu volnych kamenu:
			tiles.erase(tilesIt);
		}
		++iteraci;
		puzz.CountZeroPosition();
	}
	while(!puzz.IsSolvable() && iteraci<11);

	return;
}


int main( int argc, const char* argv[] )
{
	srand( (unsigned)time( NULL ) );

	string prikaz = "help";
	string solution, solutionOptimized = "";
	Puzzle puzz;
	SolverPRT solver1;
	SolverHad solver2;

	while(prikaz!="exit")
	{
		if(prikaz=="help")
		{
			printHelp();
		}
		else if(prikaz=="load")
		{
			loadPuzzle(puzz);
		}		
		else if(prikaz=="save")
		{
			savePuzzle(puzz);
		}		
		else if(prikaz=="new")
		{
			cout << "\nInsert n : ";
			unsigned int n = 0;
			cin >> n;
			if(n>MAX_N)
			{
				cout << "Error - n must be <= " << MAX_N << endl;
			}
			else
			{
				vector<int> tiles;
				for(unsigned int i=1; i<n*n;++i)
				{
					tiles.push_back(i);
				}
				// mezera nakonec
				tiles.push_back(0);
				puzz.SetTiles(tiles);
				puzz.SetN((unsigned)n);
			}
		}
		else if(prikaz=="rand")
		{
			cout << "\nInsert n : ";
			unsigned int n = 0;
			cin >> n;
			if(n>MAX_N)
			{
				cout << "Error - n must be <= " << MAX_N << endl;
			}
			else
			{
				randomizePuzzle(puzz,n);
			}
		}
		else if(prikaz=="print")
		{
			puzz.Print();
		}
		else if(prikaz=="rmove")
		{
			cout << "\nInsert number of moves: ";
			int moves;
			cin >> moves;
			for(int i=0; i<moves; ++i)
			{
				puzz.RandomMove();
			}
			cout << moves << " random moves done";
		}
		else if(prikaz=="solve")
		{
			solver1.SetPuzzle(puzz);
			solution.clear();
			solution = solver1.Solve();
			if(!solution.empty())
			{
				cout << "\nSolution found! " /* << solution */ << endl;
				cout << "\tlength:\t" << solution.length() << endl;
				solutionOptimized = optimizeSolution(solution);
				cout << "\nOptimized solution length:\t" << solutionOptimized.length() << endl;
			}

			// solver2.SetPuzzle(puzz);
			// solver2.Solve();

		}
		else if(prikaz=="solve2")
		{
			solver2.SetPuzzle(puzz);
			solution.clear();
			solution = solver2.Solve();
			if(!solution.empty())
			{
				cout << "\nSolution found! " /* << solution */ << endl;
				cout << "\tlength:\t" << solution.length() << endl;
				solutionOptimized = optimizeSolution(solution);
				cout << "\nOptimized solution length:\t" << solutionOptimized.length() << endl;
			}

			// solver2.SetPuzzle(puzz);
			// solver2.Solve();

		}
		else if(prikaz=="apply")
		{
			Puzzle p2 = puzz;
			cout << "\n...applying solution: " << endl;
			cout << "Solution: " << solution << endl;
			cout << "Result: ";
			p2.ApplySequence(solution);
			p2.Print();
		}
		else if(prikaz=="solution")
		{
			cout << "\nOptimized solution: " << solutionOptimized << endl;
			cout << "\tlength:\t" << solutionOptimized.length() << endl;
		}
		else if(prikaz=="dl")
		{
			cout << "\nInsert file name : ";
			string filename;
			cin >> filename;
			puzz.DirectLoad(filename);
		}
		else if(prikaz=="cls")
		{
			system("cls");
		}
		else if(prikaz=="test")
		{
			/*			
			for(int i=1; i<41; ++i)
			{
				randomizePuzzle(puzz,45);
				std::stringstream outputFile;
				outputFile << "C:\\!diplomka\\Puzzles\\2024-" << i << ".sbp";
				puzz.SaveToSbp(outputFile.str());
				cout << " " << i;

			}*/
			string filename;
			cin >> filename;
			cout << filename;

			
			// ofstream outfile ("C:\\test.txt");
			// outfile << " ";

			/*
			for(int i=1; i<41; ++i)
			{
				std::stringstream inputFile;
				inputFile << "C:\\!diplomka\\Puzzles\\2499-" << i << ".sbp";
				puzz.LoadFromSbp(inputFile.str());
				// puzz.Print();
				solver2.SetPuzzle(puzz);
				solution.clear();
				solution = solver2.Solve();
				int l2 = optimizeSolution(solution).length();
				
				solver1.SetPuzzle(puzz);
				solution.clear();
				solution = solver1.Solve();
				int l1 = optimizeSolution(solution).length();

				cout << "\n\n| l1 = " << l1 << "\tl2 = " << l2 << " |\n\n\n\n";

				outfile << "\n" << puzz.ManhattanDist() << " " << solution.length() << " " << l1 << " " << l2;
			}
			*/
			

		}
		else
		{
			cout << "\nBad command";
		}


		cout << "\n: ";
		cin >> prikaz;
	}


	return 0;

}
