#include "puzzleInputFce.h"

using namespace std;

const char * getIndent( unsigned int numIndents )
{
	static const char * pINDENT="                                      + ";
	static const unsigned int LENGTH=strlen( pINDENT );
	unsigned int n=numIndents*NUM_INDENTS_PER_SPACE;
	if ( n > LENGTH ) n = LENGTH;

	return &pINDENT[ LENGTH-n ];
}

// same as getIndent but no "+" at the end
const char * getIndentAlt( unsigned int numIndents )
{
	static const char * pINDENT="                                        ";
	static const unsigned int LENGTH=strlen( pINDENT );
	unsigned int n=numIndents*NUM_INDENTS_PER_SPACE;
	if ( n > LENGTH ) n = LENGTH;

	return &pINDENT[ LENGTH-n ];
}

int dump_attribs_to_stdout(TiXmlElement* pElement, unsigned int indent)
{
	if ( !pElement ) return 0;

	TiXmlAttribute* pAttrib=pElement->FirstAttribute();
	int i=0;
	int ival;
	double dval;
	const char* pIndent=getIndent(indent);
	printf("\n");
	while (pAttrib)
	{
		printf( "%s%s: value=[%s]", pIndent, pAttrib->Name(), pAttrib->Value());

		if (pAttrib->QueryIntValue(&ival)==TIXML_SUCCESS)    printf( " int=%d", ival);
		if (pAttrib->QueryDoubleValue(&dval)==TIXML_SUCCESS) printf( " d=%1.1f", dval);
		printf( "\n" );
		i++;
		pAttrib=pAttrib->Next();
	}
	return i;	
}



void dump_to_stdout( TiXmlNode* pParent, unsigned int indent)
{
	if ( !pParent ) return;

	TiXmlNode* pChild;
	TiXmlText* pText;
	int t = pParent->Type();
	printf( "%s", getIndent(indent));
	int num;

	switch ( t )
	{
	case TiXmlNode::TINYXML_DOCUMENT:
		printf( "Document" );
		break;

	case TiXmlNode::TINYXML_ELEMENT:
		printf( "Element [%s]", pParent->Value() );
		num=dump_attribs_to_stdout(pParent->ToElement(), indent+1);
		switch(num)
		{
			case 0:  printf( " (No attributes)"); break;
			case 1:  printf( "%s1 attribute", getIndentAlt(indent)); break;
			default: printf( "%s%d attributes", getIndentAlt(indent), num); break;
		}
		break;

	case TiXmlNode::TINYXML_COMMENT:
		printf( "Comment: [%s]", pParent->Value());
		break;

	case TiXmlNode::TINYXML_UNKNOWN:
		printf( "Unknown" );
		break;

	case TiXmlNode::TINYXML_TEXT:
		pText = pParent->ToText();
		printf( "Text: [%s]", pText->Value() );
		break;

	case TiXmlNode::TINYXML_DECLARATION:
		printf( "Declaration" );
		break;
	default:
		break;
	}
	printf( "\n" );
	for ( pChild = pParent->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
	{
		dump_to_stdout( pChild, indent+1 );
	}
}



std::string optimizeSolution(std::string solution)
{
	string optimizedSolution = "";
	bool ok = true;
	
	// znak po znaku"
	for(unsigned i=0; i<solution.size(); ++i)
	{
		char move = solution[i];
		if(optimizedSolution.empty())
		{
			ok = true;
		}
		else
		{
			switch(move)
			{
				case 'U':
						ok = optimizedSolution[optimizedSolution.length()-1] != 'D';
						break;					
				case 'D':
						ok = optimizedSolution[optimizedSolution.length()-1] != 'U';
						break;
				case 'L':
						ok = optimizedSolution[optimizedSolution.length()-1] != 'R';
						break;
				case 'R':
						ok = optimizedSolution[optimizedSolution.length()-1] != 'L';
						break;
				default:
						ok = true;

			}
		}
		if(ok)
		{
			optimizedSolution += move;
		}
		else
		{
			// vyhodit posledni znak:
			optimizedSolution.erase(optimizedSolution.length()-1);
		}

	}

	return optimizedSolution;
}
int myRand(int max, int min)
{
	if(min>max)
			return 0;
	int range = max - min;
	double rrrrand = (double) rand();
	double podil = rrrrand / (double) RAND_MAX;
	int randInt = (int)( podil * range) + min;

	return randInt;
}